# Python Connector for HubSpot #

Python Connector for HubSpot is a connectivity solution for accessing HubSpot
from Python applications to read and update data. It fully implements
the Python DB API 2.0 specification. The connector is distributed as a wheel
package for Windows and Windows Server.

Standard SQL syntax

The connector fully supports the ANSI SQL standard and lets you execute SQL
statements against your HubSpot data just like you would normally work with
relational databases. Simple queries are directly converted to HubSpot API
calls and executed on the HubSpot side.
Complex queries are transformed into simpler queries, which are then converted
to HubSpot API calls. The embedded SQL engine then processes the results
in the local cache and applies advanced SQL features from the original complex
query.

## Using the connector ##

To retrieve data from a database:

Import the module.
```
import devart.hubspot
```

Connect to a database using the `connect()` module method and obtain a `connection` object: 
If you authenticate using an API key: 
```
my_connection = devart.hubspot.connect(
    Authentication="APIKey",
    APIKey="your_api_key"
)
```
If you authenticate using an access token: 
```
my_connection = devart.hubspot.connect(
    Authentication="PrivateApp",
    AccessToken="your_access_token"
)
```
If you're using OAuth 2.0 authentication and have a refresh token:
```
my_connection = devart.hubspot.connect(
    Authentication="OAuth",
    RefreshToken="your_refresh_token"
)
```
If you're using OAuth 2.0 authentication and don't have a refresh token:
```
response = devart.hubspot.signin()
my_connection = devart.hubspot.connect(
    Authentication="OAuth",
    RefreshToken=response["Refresh Token"]
)
```

Create a `cursor` object using the `cursor()` connection method.
```
my_cursor = my_connection.cursor()
```

Execute the SQL statement using the `execute()` cursor method.
```
my_cursor.execute("SELECT * FROM employees")
```

Retrieve the result set using one of the `fetch*()` cursor methods.
```
for row in my_cursor.fetchall(): 
    print(row)
```

## Ordering and activating the license ##

You can purchase a license for the connector on the ordering page:             
https://www.devart.com/python/hubspot/ordering.html

To activate the license, follow the instructions in the documentation:             
https://docs.devart.com/python/hubspot/activate-a-license.htm

## What's new ##

Python Connector for HubSpot 1.2

- Added support for Python 3.13
- Added support for Association objects
